import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer_plugin/protocol/protocol_common.dart';
import 'package:analyzer_plugin/protocol/protocol_generated.dart';
import 'package:cat_analyzer_plugin/configs/rule_config.dart';
import 'package:cat_analyzer_plugin/fix_contributor/avoid_using_hard_coded_string_fix.dart';
import 'package:cat_analyzer_plugin/model/error_issue.dart';

AnalysisError codeIssueToAnalysisError(
    Issue issue, ResolvedUnitResult analysisResult) {
  return AnalysisError(
    issue.errorSeverity!,
    issue.errorType!,
    Location(
      analysisResult.unit?.declaredElement?.source.fullName ?? 'unknown',
      issue.offset!,
      issue.length!,
      issue.line!,
      issue.column!,
      issue.endLine!,
      issue.endColumn!,
    ),
    issue.message ?? '',
    issue.code ?? '',
    correction: issue.correction,
    hasFix: issue.hasFix,
  );
}

List<AnalysisErrorFixes> codeIssueToAnalysisErrorFixes(
    List<Issue> issueList, ResolvedUnitResult analysisResult) {
  return issueList
      .map(
        (issue) => AnalysisErrorFixes(
          codeIssueToAnalysisError(issue, analysisResult),
          fixes: [
            if (issue.correction != null)
              getSourceChangeForIssue(issue, analysisResult)
          ],
        ),
      )
      .toList();
}

Issue? locateIssueAtGivenOffset(int offset, ResolvedUnitResult analysisResult) {
  for (var rule in getAllRules()) {
    var issueList = rule.check(analysisResult) as List<Issue>;
    for (var issue in issueList) {
      if (issue.offset! <= offset &&
          (issue.offset! + issue.length!) >= offset) {
        return issue;
      }
    }
  }
  return null;
}

PrioritizedSourceChange getSourceChangeForIssue(
    Issue issue, ResolvedUnitResult analysisResult) {
  return PrioritizedSourceChange(
      99, findSourceChange(analysisResult, issue)
      );
}
